import { css, LitElement, html } from 'lit-element';

const BsAlertCss = css`
    *,
    *::before,
    *::after {
        box-sizing: border-box;
    }

    :host,
    :host::before,
    :host::after {
        box-sizing: border-box;
    }

    :host {
        display: block;
        position: relative;
        padding: 0.75rem 1.25rem;
        margin-bottom: 1rem;
        border: 1px solid transparent;
        border-radius: 0.25rem;
        opacity: 1;
        transition: opacity 0.15s linear;
    }

    :host([dismissable]) {
        padding-right: 4rem;
    }

    :host ::slotted([slot="heading"]) {
        color: inherit;
    }

    :host ::slotted([slot="dismiss"]) {
        position: absolute;
        top: 0;
        right: 0;
        padding: 0.75rem 1.25rem;
        color: inherit;
    }

    :host([context="primary"]) {
        --alert-color: var(--alert-primary-color, #004085);
        --alert-bg-color: var(--alert-primary-bg-color, #cce5ff);
        --alert-bd-color: var(--alert-primary-bd-color, #b8daff);
    }

    :host([context="secondary"]) {
        --alert-color: var(--alert-secondary-color, #383d41);
        --alert-bg-color: var(--alert-secondary-bg-color, #e2e3e5);
        --alert-bd-color: var(--alert-secondary-bd-color, #d6d8db);
    }

    :host([context="success"]) {
        --alert-color: var(--alert-success-color, #155724);
        --alert-bg-color: var(--alert-success-bg-color, #d4edda);
        --alert-bd-color: var(--alert-success-bd-color, #c3e6cb);
    }

    :host([context="info"]) {
        --alert-color: var(--alert-info-color, #0c5460);
        --alert-bg-color: var(--alert-info-bg-color, #d1ecf1);
        --alert-bd-color: var(--alert-info-bd-color, #bee5eb);
    }

    :host([context="warning"]) {
        --alert-color: var(--alert-warning-color, #856404);
        --alert-bg-color: var(--alert-warning-bg-color, #fff3cd);
        --alert-bd-color: var(--alert-warning-bd-color, #ffeeba);
    }

    :host([context="danger"]) {
        --alert-color: var(--alert-danger-color, #721c24);
        --alert-bg-color: var(--alert-danger-bg-color, #f8d7da);
        --alert-bd-color: var(--alert-danger-bd-color, #f5c6cb);
    }

    :host([context="light"]) {
        --alert-color: var(--alert-light-color, #818182);
        --alert-bg-color: var(--alert-light-bg-color, #fefefe);
        --alert-bd-color: var(--alert-light-bd-color, #fdfdfe);
    }

    :host([context="dark"]) {
        --alert-color: var(--alert-dark-color, #1b1e21);
        --alert-bg-color: var(--alert-dark-bg-color, #d6d8d9);
        --alert-bd-color: var(--alert-dark-bd-color, #c6c8ca);
    }

    :host {
        color: var(--alert-color);
        background-color: var(--alert-bg-color);
        border-color: var(--alert-bd-color);
    }
`;

/**
 * Alert component
 *
 * @element bs-alert
 *
 * @attribute dismissable - Indicates whether or not the alert can be dismissed
 * @attribute {primary|secondary|success|info|warning|danger|light|dark} context -
 * Indicates the styling context to use
 *
 * @property {Boolean} dismissable - Indicates whether or not the alert can be dismissed. default: false
 *
 * @fires close.bs.alert - Gets fired before the dismission transition has ended
 * @fires closed.bs.alert - Gets fired after the dismission transition has ended
 *
 * @slot Slot for alert content
 * @slot heading - Slot to place alert header content
 * @slot dismiss - Slot to place content that will be used for dismiss
 *
 * @cssproperty --alert-color - The alert text color. default: user agent color
 * @cssproperty --alert-bg-color - The alert background color. default: user agent color
 * @cssproperty --alert-bd-color - The border color. default: user agent color
 * @cssproperty --alert-primary-color - The primary context text color. default: #004085
 * @cssproperty --alert-primary-bg-color - The primary context background color. default: #cce5ff
 * @cssproperty --alert-primary-bd-color - The primary context border color. default: #b8daff
 * @cssproperty --alert-secondary-color - The secondary context text color. default: #383d41
 * @cssproperty --alert-secondary-bg-color - The secondary context background color. default: #e2e3e5
 * @cssproperty --alert-secondary-bd-color - The secondary context border color. default: #d6d8db
 * @cssproperty --alert-success-color - The success context text color. default: #155724
 * @cssproperty --alert-success-bg-color - The success context background color. default: #d4edda
 * @cssproperty --alert-success-bd-color - The success context border color. default: #c3e6cb
 * @cssproperty --alert-info-color - The info context text color. default: #0c5460
 * @cssproperty --alert-info-bg-color - The info context background color. default: #d1ecf1
 * @cssproperty --alert-info-bd-color - The info context border color. default: #bee5eb
 * @cssproperty --alert-warning-color - The warning context text color. default: #856404
 * @cssproperty --alert-warning-bg-color - The warning context background color. default: #fff3cd
 * @cssproperty --alert-warning-bd-color - The warning context border color. default: #ffeeba
 * @cssproperty --alert-danger-color - The danger context text color. default: #721c24
 * @cssproperty --alert-danger-bg-color - The danger context background color. default: #f8d7da
 * @cssproperty --alert-danger-bd-color - The danger context border color. default: #f5c6cb
 * @cssproperty --alert-light-color - The light context text color. default: #818182
 * @cssproperty --alert-light-bg-color - The light context background color. default: #fefefe
 * @cssproperty --alert-light-bd-color - The light context border color. default: #fdfdfe
 * @cssproperty --alert-dark-color - The dark context text color. default: #1b1e21
 * @cssproperty --alert-dark-bg-color - The dark context background color. default: #d6d8d9
 * @cssproperty --alert-dark-bd-color - The dark context border color. default: #c6c8ca
 */

class BsAlert extends LitElement {
  static get properties() {
    return {
      dismissable: {
        type: Boolean,
        reflect: true
      }
    };
  }

  static get styles() {
    return [BsAlertCss];
  }

  render() {
    return html`
            <slot name="heading"></slot>
            <slot></slot>
            <slot name="dismiss"></slot>
        `;
  }

  constructor() {
    super();
    this.dismissable = false;
  }

  firstUpdated() {
    this._addAriaRole();

    this.addEventListener("close.button.click", async _ => await this.dismiss());
  }

  _addAriaRole() {
    this.setAttribute("role", "alert");
  }
  /**
   * Dismisses an alert
   *
   * @async
   * @return {Promise} A Promise which is resolved when dismissal has finished with transition.
   */


  async dismiss() {
    if (!this.dismissable) {
      return;
    }

    return new Promise(resolve => {
      const afterTransitionEnd = _ => {
        this._fireClosedEvent();

        this.removeEventListener("transitionend", afterTransitionEnd);
        resolve();
      };

      this._fireCloseEvent();

      this.addEventListener("transitionend", afterTransitionEnd);
      requestAnimationFrame(() => this.style.opacity = "0");
    });
  }
  /**
   * Schedule updates to occur just before the next frame
   */


  async performUpdate() {
    await new Promise(resolve => requestAnimationFrame(() => resolve()));
    super.performUpdate();
  }

  _fireCloseEvent() {
    const alertCloseEvent = new CustomEvent("close.bs.alert", {
      bubbles: true,
      composed: true
    });
    this.dispatchEvent(alertCloseEvent);
  }

  _fireClosedEvent() {
    const alertClosedEvent = new CustomEvent("closed.bs.alert", {
      bubbles: true,
      composed: true
    });
    this.dispatchEvent(alertClosedEvent);
  }

}
if (!customElements.get("bs-alert")) customElements.define("bs-alert", BsAlert);

const BsAlertHrCss = css`
    :host([context="primary"]) {
        --alert-hr-bd-top-color: var(--alert-hr-primary-bd-top-color, #9fcdff);
    }

    :host([context="secondary"]) {
        --alert-hr-bd-top-color: var(
            --alert-hr-secondary-bd-top-color,
            #c8cbcf
        );
    }

    :host([context="success"]) {
        --alert-hr-bd-top-color: var(--alert-hr-success-bd-top-color, #b1dfbb);
    }

    :host([context="info"]) {
        --alert-hr-bd-top-color: var(--alert-hr-info-bd-top-color, #abdde5);
    }

    :host([context="warning"]) {
        --alert-hr-bd-top-color: var(--alert-hr-warning-bd-top-color, #ffe8a1);
    }

    :host([context="danger"]) {
        --alert-hr-bd-top-color: var(--alert-hr-danger-bd-top-color, #f1b0b7);
    }

    :host([context="light"]) {
        --alert-hr-bd-top-color: var(--alert-hr-light-bd-top-color, #ececf6);
    }

    :host([context="dark"]) {
        --alert-hr-bd-top-color: var(--alert-hr-dark-bd-top-color, #b9bbbe);
    }

    hr {
        height: 0;
        border: 0;
        overflow: visible;
        box-sizing: content-box;
        margin-top: 1rem;
        margin-bottom: 1rem;
        border-top-width: 1px;
        border-top-style: solid;
        border-top-color: var(--alert-hr-bd-top-color, rgba(0, 0, 0, 0.1));
    }
`;

/**
 * Alert hr component
 *
 * @element bs-alert-hr
 *
 * @attribute {primary|secondary|success|info|warning|danger|light|dark} context -
 * Indicates the styling context to use
 *
 * @cssproperty --alert-hr-bd-top-color - The hr top border color. default: rgba(0, 0, 0, 0.1)
 * @cssproperty --alert-hr-primary-bd-top-color - The hr primary context top border color. default: #9fcdff
 * @cssproperty --alert-hr-secondary-bd-top-color - The hr secondary context top border color. default: #c8cbcf
 * @cssproperty --alert-hr-success-bd-top-color - The hr success context top border color. default: #b1dfbb
 * @cssproperty --alert-hr-info-bd-top-color - The hr info context top border color. default: #abdde5
 * @cssproperty --alert-hr-warning-bd-top-color - The hr warning context top border color. default: #ffe8a1
 * @cssproperty --alert-hr-danger-bd-top-color - The hr danger context top border color. default: #f1b0b7
 * @cssproperty --alert-hr-light-bd-top-color - The hr light context top border color. default: #ececf6
 * @cssproperty --alert-hr-dark-bd-top-color - The hr dark context top border color. default: #b9bbbe
 */

class BsAlertHr extends LitElement {
  static get styles() {
    return [BsAlertHrCss];
  }

  render() {
    return html`
            <hr />
        `;
  }

}
if (!window.customElements.get("bs-alert-hr")) window.customElements.define("bs-alert-hr", BsAlertHr);

const BsAlertLinkCss = css`
    :host([context="primary"]) {
        --alert-link-color: var(--alert-link-primary-color, #002752);
    }

    :host([context="secondary"]) {
        --alert-link-color: var(--alert-link-secondary-color, #202326);
    }

    :host([context="success"]) {
        --alert-link-color: var(--alert-link-success-color, #0b2e13);
    }

    :host([context="info"]) {
        --alert-link-color: var(--alert-link-info-color, #062c33);
    }

    :host([context="warning"]) {
        --alert-link-color: var(--alert-link-warning-color, #533f03);
    }

    :host([context="danger"]) {
        --alert-link-color: var(--alert-link-danger-color, #491217);
    }

    :host([context="light"]) {
        --alert-link-color: var(--alert-link-light-color, #686868);
    }

    :host([context="dark"]) {
        --alert-link-color: var(--alert-link-dark-color, #040505);
    }

    a {
        color: var(--alert-link-color, #007bff);
        font-weight: var(--alert-link-font-weight, 700);
        text-decoration: var(--alert-link-text-decoration, none);
        background-color: var(--alert-link-bg-color, transparent);
        -webkit-text-decoration-skip: objects;
    }

    a:hover {
        color: var(--alert-link-hover-color, #0056b3);
        text-decoration: var(--alert-link-hover-text-decoration, underline);
    }
`;

/**
 * Alert link component
 *
 * @element bs-alert-link
 *
 * @attribute href - The hyper link to point to
 * @attribute target - The context target
 * @attribute {primary|secondary|success|info|warning|danger|light|dark} context -
 * Indicates the styling context to use
 *
 * @property {String} href - The hyper link to point to
 * @property {String} target - The context target. default: _self
 *
 * @slot Slot for alert link content
 *
 * @cssproperty --alert-link-color - The link color. default: #007bff
 * @cssproperty --alert-link-font-weight - The link font weight. default 700
 * @cssproperty --alert-link-text-decoration - The link text decoration. default: none
 * @cssproperty --alert-link-bg-color - The link background color. default: transparent
 * @cssproperty --alert-link-hover-color - The link hover color. default: #0056b3
 * @cssproperty --alert-link-hover-text-decoration - The link hover text decoration. default: underline
 * @cssproperty --alert-link-primary-color - The primary context link color. defaul: #002752
 * @cssproperty --alert-link-secondary-color - The secondary context link color. default: #202326
 * @cssproperty --alert-link-success-color - The success context link color. default: #0b2e13
 * @cssproperty --alert-link-info-color - The info context link color. default: #062c33
 * @cssproperty --alert-link-warning-color - The warning context link color, default: #533f03
 * @cssproperty --alert-link-danger-color - The danger context link color. default: #491217
 * @cssproperty --alert-link-light-color - The light context link color. default: #686868
 * @cssproperty --alert-link-dark-color - The dark context link color. default: #040505
 */

class BsAlertLink extends LitElement {
  static get properties() {
    return {
      href: {
        type: String,
        reflect: true
      },
      target: {
        type: String,
        reflect: true
      }
    };
  }

  static get styles() {
    return [BsAlertLinkCss];
  }

  render() {
    return html`
            <a href="${this.href}" .target="${this.target}">
                <slot></slot>
            </a>
        `;
  }

  constructor() {
    super();
    this.href = '';
    this.target = '_self';
  }

}
if (!window.customElements.get("bs-alert-link")) window.customElements.define("bs-alert-link", BsAlertLink);

const BsAlertDismissCss = css`
    .close {
        border: 0;
        float: right;
        font-size: 1.5rem;
        font-weight: 700;
        line-height: 1;
        color: #000;
        text-shadow: 0 1px 0 #fff;
        opacity: 0.5;
        background-color: transparent;
        -webkit-appearance: none;
        top: 0;
        right: 0;
        color: inherit;
        position: absolute;
        padding: 0.75rem 1.25rem;
    }

    .close:not(:disabled):not(.disabled):hover,
    .close:not(:disabled):not(.disabled):focus {
        color: #000;
        text-decoration: none;
        opacity: 0.75;
        cursor: pointer;
    }
`;

/**
 * Alert dismiss component
 *
 * @element bs-alert-dismiss
 *
 * @fires close.button.click - Fired when the button is clicked
 */

class BsAlertDismiss extends LitElement {
  static get styles() {
    return BsAlertDismissCss;
  }

  render() {
    return html`
            <button type="button" class="close" aria-label="Close">
                <span aria-hidden="true">&times;</span>
            </button>
        `;
  }

  firstUpdated() {
    const buttonElement = this.shadowRoot.querySelector("button");
    buttonElement.addEventListener("click", _ => this._handleClick());
  }

  _handleClick() {
    const closeButtonClickEvent = new CustomEvent("close.button.click", {
      bubbles: true,
      composed: true
    });
    this.dispatchEvent(closeButtonClickEvent);
  }

}
if (!window.customElements.get("bs-alert-dismiss")) window.customElements.define("bs-alert-dismiss", BsAlertDismiss);

export { BsAlert, BsAlertCss, BsAlertDismiss, BsAlertDismissCss, BsAlertHr, BsAlertHrCss, BsAlertLink, BsAlertLinkCss };
//# sourceMappingURL=index.js.map
